window.Gleam.initSearch([{"doc":"themis","title":"themis","content":"# Themis\n\nPrometheus client in pure Gleam!\n\n[![Package Version](https://img.shields.io/hexpm/v/themis)](https://hex.pm/packages/themis)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/themis/)\n\n```sh\ngleam add themis\n```\n\n## Quick Start\n\n```gleam\nimport gleam/dict\nimport gleam/io\nimport gleam/set\nimport themis\nimport themis/counter\nimport themis/gauge\nimport themis/histogram\nimport themis/number\n\npub fn main() {\n  // Create the new metrics store\n  let metrics_store = themis.new()\n\n  // Gauge\n\n  // This can fail if the metric name is invalid\n  let assert Ok(metrics_store) =\n    gauge.register(\n      metrics_store,\n      &quot;my_first_metric&quot;,\n      &quot;A gauge Prometheus metric&quot;,\n    )\n\n  let labels = dict.from_list([#(&quot;foo&quot;, &quot;bar&quot;)])\n  let value = number.integer(10)\n  let assert Ok(metrics_store) =\n    gauge.observe(metrics_store, &quot;my_first_metric&quot;, labels, value)\n\n  // Counter\n\n  let assert Ok(metrics_store) =\n    counter.register(\n      metrics_store,\n      &quot;my_second_metric&quot;,\n      &quot;A counter Prometheus metric&quot;,\n    )\n\n  let labels = dict.from_list([#(&quot;wibble&quot;, &quot;wobble&quot;)])\n  let other_labels = dict.from_list([#(&quot;wii&quot;, &quot;woo&quot;)])\n  // Counters can optionally be initialized manually\n  let assert Ok(metrics_store) =\n    counter.init_record(metrics_store, &quot;my_second_metric&quot;, labels)\n  let assert Ok(metrics_store) =\n    counter.increment(metrics_store, &quot;my_second_metric&quot;, labels)\n  // When incrementing a counter with new labels, a new counter will automatically be initialized then incremented\n  let assert Ok(metrics_store) =\n    counter.increment_by(\n      metrics_store,\n      &quot;my_second_metric&quot;,\n      other_labels,\n      number.decimal(1.2),\n    )\n\n  // Histogram\n\n  // Histograms work with buckets. Each bucket needs an upper boundary.\n  let buckets =\n    set.from_list([\n      number.decimal(0.05),\n      number.decimal(0.1),\n      number.decimal(0.25),\n      number.decimal(0.5),\n      number.integer(1),\n    ])\n  let assert Ok(metrics_store) =\n    histogram.register(\n      metrics_store,\n      &quot;my_third_metric&quot;,\n      &quot;A histogram Prometheus metric&quot;,\n      buckets,\n    )\n\n  let value = number.integer(20)\n  let other_value = number.decimal(1.5)\n  let labels = dict.from_list([#(&quot;toto&quot;, &quot;tata&quot;)])\n  let other_labels = dict.from_list([#(&quot;toto&quot;, &quot;titi&quot;)])\n  // Histograms can optionally be initialized manually\n  let assert Ok(metrics_store) =\n    histogram.observe(metrics_store, &quot;my_third_metric&quot;, labels, value)\n  // When incrementing a histogram with new labels, a new histogram will automatically be initialized\n  let assert Ok(metrics_store) =\n    histogram.observe(\n      metrics_store,\n      &quot;my_third_metric&quot;,\n      other_labels,\n      other_value,\n    )\n\n  // Printing all the metrics as a Prometheus-scrapable String\n  themis.print(metrics_store)\n  |&gt; io.println\n}\n\n```\n\nThis code will print the following prometheus-compatible metrics:\n\n```\n# HELP my_first_metric A gauge Prometheus metric\n# TYPE my_first_metric gauge\nmy_first_metric{foo=&quot;bar&quot;} 10\n\n# HELP my_second_metric_total A counter Prometheus metric\n# TYPE my_second_metric_total counter\nmy_second_metric_total{wibble=&quot;wobble&quot;} 1\nmy_second_metric_total{wii=&quot;woo&quot;} 1.2\n\n# HELP my_third_metric A histogram Prometheus metric\n# TYPE my_third_metric histogram\nmy_third_metric_bucket{le=&quot;0.05&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;0.1&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;0.25&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;0.5&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;1&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;+Inf&quot;,toto=&quot;tata&quot;} 1\nmy_third_metric_sum{toto=&quot;tata&quot;} 1\nmy_third_metric_count{toto=&quot;tata&quot;} 1\n\nmy_third_metric_bucket{le=&quot;0.05&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;0.1&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;0.25&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;0.5&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;1&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;+Inf&quot;,toto=&quot;titi&quot;} 1\nmy_third_metric_sum{toto=&quot;titi&quot;} 1\nmy_third_metric_count{toto=&quot;titi&quot;} 1\n\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/themis&gt;.\n\n## Usage\n\n### Working with Different Numeric Types\n\nThemis metric values are set using the dedicated `Number` type. There are 5 number types available:\n\n```gleam\nimport themis/number\n\n// Integer values\nlet integer = number.integer(1_234_567)\n\n// Decimal (float) values\nlet decimal = number.decimal(23.5)\n\n// Special values\nlet positive_infinity = number.positive_infinity()\nlet negative_infinity = number.negative_infinity()\nlet not_a_number = number.not_a_number()\n```\n### Metric Types\n\n#### Gauges\n\nGauges are metrics that represent a single numerical value that can arbitrarily go up and down. They are typically used for measured values like temperatures, current memory usage, or number of active connections.\n\n```gleam\nimport themis/gauge\n\n// Register a new gauge metric\nlet assert Ok(metrics_store) = \n  gauge.register(\n    metrics_store,\n    &quot;process_memory_bytes&quot;,\n    &quot;Current memory usage in bytes&quot;,\n  )\n\n// Set a gauge value with labels\nlet labels = dict.from_list([#(&quot;process&quot;, &quot;web_server&quot;)])\nlet value = number.integer(52_428_800)  // 50MB in bytes\nlet assert Ok(metrics_store) =\n  gauge.observe(metrics_store, &quot;process_memory_bytes&quot;, labels, value)\n```\n\n#### Counters\n\nCounters are cumulative metrics that can only increase or be reset to zero. They are typically used to count requests served, tasks completed, errors occurred, or other countable occurrences.\n\n```gleam\nimport themis/counter\n\n// Register a new counter metric\nlet assert Ok(metrics_store) =\n  counter.register(\n    metrics_store,\n    &quot;http_requests_total&quot;,\n    &quot;Total number of HTTP requests made&quot;,\n  )\n\n// Initialize a counter with specific labels (optional)\nlet labels = dict.from_list([#(&quot;method&quot;, &quot;GET&quot;), #(&quot;path&quot;, &quot;/api/users&quot;)])\nlet assert Ok(metrics_store) =\n  counter.init_record(metrics_store, &quot;http_requests_total&quot;, labels)\n\n// Increment counter by 1\nlet assert Ok(metrics_store) =\n  counter.increment(metrics_store, &quot;http_requests_total&quot;, labels)\n\n// Increment counter by specific amount\nlet assert Ok(metrics_store) =\n  counter.increment_by(\n    metrics_store,\n    &quot;http_requests_total&quot;,\n    labels,\n    number.decimal(5.0),\n  )\n```\n\n#### Histograms\n\nHistograms sample observations (usually duration or response size) and count them in configurable buckets. They also provide a sum of all observed values and a count of observations.\n\n```gleam\nimport themis/histogram\n\n// Define histogram buckets (upper bounds of observation buckets in seconds)\nlet buckets =\n  set.from_list([\n    number.decimal(0.005),  // 5ms\n    number.decimal(0.01),   // 10ms\n    number.decimal(0.025),  // 25ms\n    number.decimal(0.05),   // 50ms\n    number.decimal(0.1),    // 100ms\n    number.decimal(0.25),   // 250ms\n    number.decimal(0.5),    // 500ms\n    number.decimal(1.0),    // 1s\n  ])\n\n// Register a new histogram metric\nlet assert Ok(metrics_store) =\n  histogram.register(\n    metrics_store,\n    &quot;http_request_duration_seconds&quot;,\n    &quot;HTTP request duration in seconds&quot;,\n    buckets,\n  )\n\n// Record an observation\nlet labels = dict.from_list([#(&quot;method&quot;, &quot;POST&quot;), #(&quot;path&quot;, &quot;/api/users&quot;)])\nlet duration = number.decimal(0.157)  // 157ms\nlet assert Ok(metrics_store) =\n  histogram.observe(\n    metrics_store,\n    &quot;http_request_duration_seconds&quot;,\n    labels,\n    duration,\n  )\n```\n\nEach histogram observation is counted in all buckets with upper bounds greater than the observation value. The `+Inf` bucket is automatically added and counts all observations. Additionally, histograms track the sum of all observed values and the total count of observations.\n\n#### Summaries\n\nSummaries have not yet been implemented, because at first glance it seems an accurate summary must keep a complete history of all the observed values, which will be a huge memory hog. This means I would have to implement some algorithm that goes way above my head to instead derive an approximation. I ain&#39;t doin&#39; that (maybe one day if I need summaries but don&#39;t hold your breath).&lt;br&gt;\nIf you&#39;re feeling adventurous, feel free to open a PR.\n\n## License\n\nMIT","url":"index.html"},{"doc":"themis/internal/label","title":"LabelError","content":"pub type LabelError {\n  InvalidLabelName\n}\n\nInvalidLabelName\n\n\nSynonyms:\nthemis/internal/label.LabelError\nthemis/internal/label LabelError","url":"themis/internal/label.html#LabelError"},{"doc":"themis/internal/label","title":"LabelName","content":"pub opaque type LabelName\n\n\nSynonyms:\nthemis/internal/label.LabelName\nthemis/internal/label LabelName","url":"themis/internal/label.html#LabelName"},{"doc":"themis/internal/label","title":"LabelSet","content":"pub opaque type LabelSet\n\n\nSynonyms:\nthemis/internal/label.LabelSet\nthemis/internal/label LabelSet","url":"themis/internal/label.html#LabelSet"},{"doc":"themis/internal/label","title":"add_label","content":"pub fn add_label(\n  labels labels: LabelSet,\n  new_label key: String,\n  new_label_value value: String,\n) -&gt; Result(LabelSet, LabelError)\n\nSynonyms:\nthemis/internal/label.add_label\nthemis/internal/label add_label","url":"themis/internal/label.html#add_label"},{"doc":"themis/internal/label","title":"delete_label","content":"pub fn delete_label(\n  labels labels: LabelSet,\n  label_to_remove key: String,\n) -&gt; Result(LabelSet, LabelError)\n\nSynonyms:\nthemis/internal/label.delete_label\nthemis/internal/label delete_label","url":"themis/internal/label.html#delete_label"},{"doc":"themis/internal/label","title":"from_dict","content":"pub fn from_dict(\n  labels labels: Dict(String, String),\n) -&gt; Result(LabelSet, LabelError)\n\nSynonyms:\nthemis/internal/label.from_dict\nthemis/internal/label from_dict","url":"themis/internal/label.html#from_dict"},{"doc":"themis/internal/label","title":"is_valid_label","content":"pub fn is_valid_label(label: String) -&gt; Bool\n\nSynonyms:\nthemis/internal/label.is_valid_label\nthemis/internal/label is_valid_label","url":"themis/internal/label.html#is_valid_label"},{"doc":"themis/internal/label","title":"new","content":"pub fn new() -&gt; LabelSet\n\nSynonyms:\nthemis/internal/label.new\nthemis/internal/label new","url":"themis/internal/label.html#new"},{"doc":"themis/internal/label","title":"print","content":"pub fn print(labels labels: LabelSet) -&gt; String\n\nSynonyms:\nthemis/internal/label.print\nthemis/internal/label print","url":"themis/internal/label.html#print"},{"doc":"themis/internal/label","title":"themis/internal/label","content":"","url":"themis/internal/label.html"},{"doc":"themis/internal/metric","title":"Metric","content":"pub type Metric(kind, record_type, extra) {\n  Metric(\n    description: String,\n    records: Dict(label.LabelSet, record_type),\n    extra: extra,\n  )\n}\n\nMetric(\n  description: String,\n  records: Dict(label.LabelSet, record_type),\n  extra: extra,\n)\n\n\nSynonyms:\nthemis/internal/metric.Metric\nthemis/internal/metric Metric","url":"themis/internal/metric.html#Metric"},{"doc":"themis/internal/metric","title":"MetricError","content":"pub type MetricError {\n  InvalidMetricName\n  InvalidWordInName(word: String)\n}\n\nInvalidMetricName\n\n\nInvalidWordInName(word: String)\n\n\nSynonyms:\nthemis/internal/metric.MetricError\nthemis/internal/metric MetricError","url":"themis/internal/metric.html#MetricError"},{"doc":"themis/internal/metric","title":"MetricName","content":"pub opaque type MetricName\n\n\nSynonyms:\nthemis/internal/metric.MetricName\nthemis/internal/metric MetricName","url":"themis/internal/metric.html#MetricName"},{"doc":"themis/internal/metric","title":"is_valid_name","content":"pub fn is_valid_name(name: String) -&gt; Bool\n\nSynonyms:\nthemis/internal/metric.is_valid_name\nthemis/internal/metric is_valid_name","url":"themis/internal/metric.html#is_valid_name"},{"doc":"themis/internal/metric","title":"name_to_string","content":"pub fn name_to_string(from from: MetricName) -&gt; String\n\nSynonyms:\nthemis/internal/metric.name_to_string\nthemis/internal/metric name_to_string","url":"themis/internal/metric.html#name_to_string"},{"doc":"themis/internal/metric","title":"new_name","content":"pub fn new_name(\n  from: String,\n  blacklist: List(String),\n) -&gt; Result(MetricName, MetricError)\n\nSynonyms:\nthemis/internal/metric.new_name\nthemis/internal/metric new_name","url":"themis/internal/metric.html#new_name"},{"doc":"themis/internal/metric","title":"themis/internal/metric","content":"","url":"themis/internal/metric.html"},{"doc":"themis/number","title":"ComparisonError","content":"pub type ComparisonError {\n  NaNValue\n}\n\nNaNValue\n\n\nSynonyms:\nthemis/number.ComparisonError\nthemis/number ComparisonError","url":"themis/number.html#ComparisonError"},{"doc":"themis/number","title":"Number","content":"pub type Number {\n  PosInf\n  NegInf\n  NaN\n  Dec(Float)\n  Int(Int)\n}\n Prometheus numbers.\n Helpful for displaying and adding/comparing values\n of different types.\nPosInf\n\n\nNegInf\n\n\nNaN\n\n\nDec(Float)\n\n\nInt(Int)\n\n\nSynonyms:\nthemis/number.Number\nthemis/number Number","url":"themis/number.html#Number"},{"doc":"themis/number","title":"add","content":"pub fn add(value1 val1: Number, value2 val2: Number) -&gt; Number\n Add two numbers.\n Any `NaN` input value will always return `NaN`.\n `PosInf` + `NegInf` = `NaN`\nSynonyms:\nthemis/number.add\nthemis/number add","url":"themis/number.html#add"},{"doc":"themis/number","title":"compare","content":"pub fn compare(\n  compare val1: Number,\n  to val2: Number,\n) -&gt; Result(Order, ComparisonError)\n Compare two numbers (see gleam_stdlib/order)\n Will return an error if either value is NaN\nSynonyms:\nthemis/number.compare\nthemis/number compare","url":"themis/number.html#compare"},{"doc":"themis/number","title":"decimal","content":"pub fn decimal(value value: Float) -&gt; Number\n Creates a Number representing a decimal value.\nSynonyms:\nthemis/number.decimal\nthemis/number decimal","url":"themis/number.html#decimal"},{"doc":"themis/number","title":"integer","content":"pub fn integer(value value: Int) -&gt; Number\n Creates a Number representing an integer value.\nSynonyms:\nthemis/number.integer\nthemis/number integer","url":"themis/number.html#integer"},{"doc":"themis/number","title":"negative_infinity","content":"pub fn negative_infinity() -&gt; Number\n Creates a Number representing negative infinity.\nSynonyms:\nthemis/number.negative_infinity\nthemis/number negative_infinity","url":"themis/number.html#negative_infinity"},{"doc":"themis/number","title":"not_a_number","content":"pub fn not_a_number() -&gt; Number\n Creates a Number representing NaN (Not a Number).\nSynonyms:\nthemis/number.not_a_number\nthemis/number not_a_number","url":"themis/number.html#not_a_number"},{"doc":"themis/number","title":"positive_infinity","content":"pub fn positive_infinity() -&gt; Number\n Creates a Number representing positive infinity.\nSynonyms:\nthemis/number.positive_infinity\nthemis/number positive_infinity","url":"themis/number.html#positive_infinity"},{"doc":"themis/number","title":"print","content":"pub fn print(number: Number) -&gt; String\n Prometheus-scrapable representation of a number\nSynonyms:\nthemis/number.print\nthemis/number print","url":"themis/number.html#print"},{"doc":"themis/number","title":"themis/number","content":"","url":"themis/number.html"},{"doc":"themis/internal/metric/counter","title":"Counter","content":"pub type Counter\n\n\nSynonyms:\nthemis/internal/metric/counter.Counter\nthemis/internal/metric/counter Counter","url":"themis/internal/metric/counter.html#Counter"},{"doc":"themis/internal/metric/counter","title":"CounterError","content":"pub type CounterError {\n  RecordAlreadyExists\n  RecordNotFound\n  NameError(metric.MetricError)\n}\n\nRecordAlreadyExists\n\n\nRecordNotFound\n\n\nNameError(metric.MetricError)\n\n\nSynonyms:\nthemis/internal/metric/counter.CounterError\nthemis/internal/metric/counter CounterError","url":"themis/internal/metric/counter.html#CounterError"},{"doc":"themis/internal/metric/counter","title":"delete_record","content":"pub fn delete_record(\n  from from: Metric(Counter, Number, Nil),\n  labels labels: LabelSet,\n) -&gt; Metric(Counter, Number, Nil)\n\nSynonyms:\nthemis/internal/metric/counter.delete_record\nthemis/internal/metric/counter delete_record","url":"themis/internal/metric/counter.html#delete_record"},{"doc":"themis/internal/metric/counter","title":"increment","content":"pub fn increment(\n  from from: Metric(Counter, Number, Nil),\n  labels labels: LabelSet,\n) -&gt; Metric(Counter, Number, Nil)\n\nSynonyms:\nthemis/internal/metric/counter.increment\nthemis/internal/metric/counter increment","url":"themis/internal/metric/counter.html#increment"},{"doc":"themis/internal/metric/counter","title":"increment_by","content":"pub fn increment_by(\n  from from: Metric(Counter, Number, Nil),\n  labels labels: LabelSet,\n  by by: Number,\n) -&gt; Metric(Counter, Number, Nil)\n\nSynonyms:\nthemis/internal/metric/counter.increment_by\nthemis/internal/metric/counter increment_by","url":"themis/internal/metric/counter.html#increment_by"},{"doc":"themis/internal/metric/counter","title":"init_record","content":"pub fn init_record(\n  from from: Metric(Counter, Number, Nil),\n  with_labels labels: LabelSet,\n) -&gt; Metric(Counter, Number, Nil)\n\nSynonyms:\nthemis/internal/metric/counter.init_record\nthemis/internal/metric/counter init_record","url":"themis/internal/metric/counter.html#init_record"},{"doc":"themis/internal/metric/counter","title":"new","content":"pub fn new(\n  name name: String,\n  description description: String,\n) -&gt; Result(\n  #(MetricName, Metric(Counter, Number, Nil)),\n  MetricError,\n)\n\nSynonyms:\nthemis/internal/metric/counter.new\nthemis/internal/metric/counter new","url":"themis/internal/metric/counter.html#new"},{"doc":"themis/internal/metric/counter","title":"new_name","content":"pub fn new_name(\n  name name: String,\n) -&gt; Result(MetricName, MetricError)\n\nSynonyms:\nthemis/internal/metric/counter.new_name\nthemis/internal/metric/counter new_name","url":"themis/internal/metric/counter.html#new_name"},{"doc":"themis/internal/metric/counter","title":"print","content":"pub fn print(\n  metric metric: Metric(Counter, Number, Nil),\n  name name: MetricName,\n) -&gt; String\n\nSynonyms:\nthemis/internal/metric/counter.print\nthemis/internal/metric/counter print","url":"themis/internal/metric/counter.html#print"},{"doc":"themis/internal/metric/counter","title":"themis/internal/metric/counter","content":"","url":"themis/internal/metric/counter.html"},{"doc":"themis/internal/metric/gauge","title":"Gauge","content":"pub type Gauge\n\n\nSynonyms:\nthemis/internal/metric/gauge.Gauge\nthemis/internal/metric/gauge Gauge","url":"themis/internal/metric/gauge.html#Gauge"},{"doc":"themis/internal/metric/gauge","title":"GaugeError","content":"pub type GaugeError {\n  NameError(metric.MetricError)\n}\n\nNameError(metric.MetricError)\n\n\nSynonyms:\nthemis/internal/metric/gauge.GaugeError\nthemis/internal/metric/gauge GaugeError","url":"themis/internal/metric/gauge.html#GaugeError"},{"doc":"themis/internal/metric/gauge","title":"delete_record","content":"pub fn delete_record(\n  from from: Metric(Gauge, Number, Nil),\n  labels labels: LabelSet,\n) -&gt; Metric(Gauge, Number, Nil)\n\nSynonyms:\nthemis/internal/metric/gauge.delete_record\nthemis/internal/metric/gauge delete_record","url":"themis/internal/metric/gauge.html#delete_record"},{"doc":"themis/internal/metric/gauge","title":"new","content":"pub fn new(\n  name name: String,\n  description description: String,\n) -&gt; Result(\n  #(MetricName, Metric(Gauge, Number, Nil)),\n  MetricError,\n)\n\nSynonyms:\nthemis/internal/metric/gauge.new\nthemis/internal/metric/gauge new","url":"themis/internal/metric/gauge.html#new"},{"doc":"themis/internal/metric/gauge","title":"new_name","content":"pub fn new_name(\n  name name: String,\n) -&gt; Result(MetricName, MetricError)\n\nSynonyms:\nthemis/internal/metric/gauge.new_name\nthemis/internal/metric/gauge new_name","url":"themis/internal/metric/gauge.html#new_name"},{"doc":"themis/internal/metric/gauge","title":"observe","content":"pub fn observe(\n  to to: Metric(Gauge, Number, Nil),\n  labels labels: LabelSet,\n  value value: Number,\n) -&gt; Metric(Gauge, Number, Nil)\n\nSynonyms:\nthemis/internal/metric/gauge.observe\nthemis/internal/metric/gauge observe","url":"themis/internal/metric/gauge.html#observe"},{"doc":"themis/internal/metric/gauge","title":"print","content":"pub fn print(\n  metric metric: Metric(Gauge, Number, Nil),\n  name name: MetricName,\n) -&gt; String\n\nSynonyms:\nthemis/internal/metric/gauge.print\nthemis/internal/metric/gauge print","url":"themis/internal/metric/gauge.html#print"},{"doc":"themis/internal/metric/gauge","title":"themis/internal/metric/gauge","content":"","url":"themis/internal/metric/gauge.html"},{"doc":"themis/internal/metric/histogram","title":"Histogram","content":"pub type Histogram\n\n\nSynonyms:\nthemis/internal/metric/histogram.Histogram\nthemis/internal/metric/histogram Histogram","url":"themis/internal/metric/histogram.html#Histogram"},{"doc":"themis/internal/metric/histogram","title":"HistogramError","content":"pub type HistogramError {\n  InvalidNaNLabel\n  NumberError(number.ComparisonError)\n  MetricError(metric.MetricError)\n}\n\nInvalidNaNLabel\n\n\nNumberError(number.ComparisonError)\n\n\nMetricError(metric.MetricError)\n\n\nSynonyms:\nthemis/internal/metric/histogram.HistogramError\nthemis/internal/metric/histogram HistogramError","url":"themis/internal/metric/histogram.html#HistogramError"},{"doc":"themis/internal/metric/histogram","title":"HistogramRecord","content":"pub type HistogramRecord {\n  HistogramRecord(\n    count: Number,\n    sum: Number,\n    buckets: Dict(Number, Number),\n  )\n}\n\nHistogramRecord(\n  count: Number,\n  sum: Number,\n  buckets: Dict(Number, Number),\n)\n\n\nSynonyms:\nthemis/internal/metric/histogram.HistogramRecord\nthemis/internal/metric/histogram HistogramRecord","url":"themis/internal/metric/histogram.html#HistogramRecord"},{"doc":"themis/internal/metric/histogram","title":"delete_record","content":"pub fn delete_record(\n  from from: Metric(Histogram, HistogramRecord, Set(Number)),\n  labels labels: LabelSet,\n) -&gt; Metric(Histogram, HistogramRecord, Set(Number))\n\nSynonyms:\nthemis/internal/metric/histogram.delete_record\nthemis/internal/metric/histogram delete_record","url":"themis/internal/metric/histogram.html#delete_record"},{"doc":"themis/internal/metric/histogram","title":"init_record","content":"pub fn init_record(\n  to to: Metric(Histogram, HistogramRecord, Set(Number)),\n  labels labels: LabelSet,\n) -&gt; Metric(Histogram, HistogramRecord, Set(Number))\n\nSynonyms:\nthemis/internal/metric/histogram.init_record\nthemis/internal/metric/histogram init_record","url":"themis/internal/metric/histogram.html#init_record"},{"doc":"themis/internal/metric/histogram","title":"new","content":"pub fn new(\n  name name: String,\n  description description: String,\n  buckets buckets: Set(Number),\n) -&gt; Result(\n  #(MetricName, Metric(Histogram, HistogramRecord, Set(Number))),\n  HistogramError,\n)\n\nSynonyms:\nthemis/internal/metric/histogram.new\nthemis/internal/metric/histogram new","url":"themis/internal/metric/histogram.html#new"},{"doc":"themis/internal/metric/histogram","title":"new_name","content":"pub fn new_name(\n  name name: String,\n) -&gt; Result(MetricName, MetricError)\n\nSynonyms:\nthemis/internal/metric/histogram.new_name\nthemis/internal/metric/histogram new_name","url":"themis/internal/metric/histogram.html#new_name"},{"doc":"themis/internal/metric/histogram","title":"observe","content":"pub fn observe(\n  to to: Metric(Histogram, HistogramRecord, Set(Number)),\n  labels labels: LabelSet,\n  observed value: Number,\n) -&gt; Metric(Histogram, HistogramRecord, Set(Number))\n\nSynonyms:\nthemis/internal/metric/histogram.observe\nthemis/internal/metric/histogram observe","url":"themis/internal/metric/histogram.html#observe"},{"doc":"themis/internal/metric/histogram","title":"print","content":"pub fn print(\n  metric metric: Metric(Histogram, HistogramRecord, Set(Number)),\n  name name: MetricName,\n) -&gt; String\n\nSynonyms:\nthemis/internal/metric/histogram.print\nthemis/internal/metric/histogram print","url":"themis/internal/metric/histogram.html#print"},{"doc":"themis/internal/metric/histogram","title":"themis/internal/metric/histogram","content":"","url":"themis/internal/metric/histogram.html"},{"doc":"themis","title":"Store","content":"pub type Store {\n  Store(\n    gauges: Dict(metric.MetricName, Metric(Gauge, Number, Nil)),\n    counters: Dict(\n      metric.MetricName,\n      Metric(Counter, Number, Nil),\n    ),\n    histograms: Dict(\n      metric.MetricName,\n      Metric(Histogram, HistogramRecord, Set(Number)),\n    ),\n  )\n}\n A Store manages a collection of metrics, ensuring unique metric names.\nStore(\n  gauges: Dict(metric.MetricName, Metric(Gauge, Number, Nil)),\n  counters: Dict(metric.MetricName, Metric(Counter, Number, Nil)),\n  histograms: Dict(\n    metric.MetricName,\n    Metric(Histogram, HistogramRecord, Set(Number)),\n  ),\n)\n\n\nSynonyms:\nthemis.Store\nthemis Store","url":"themis.html#Store"},{"doc":"themis","title":"StoreError","content":"pub type StoreError {\n  MetricNameAlreadyInUse\n  MetricNameNotFound\n  MetricError(metric.MetricError)\n  LabelError(label.LabelError)\n  CounterError(counter.CounterError)\n  HistogramError(histogram.HistogramError)\n}\n Represents possible errors that can occur when operating on the Store.\nMetricNameAlreadyInUse\n Returned when attempting to create a metric with a name that&#39;s already in use\n\nMetricNameNotFound\n Returned when attempting to operate on a metric that doesn&#39;t exist\n\nMetricError(metric.MetricError)\n Wraps errors related to metric name validation\n\nLabelError(label.LabelError)\n Wraps errors related to label validation\n\nCounterError(counter.CounterError)\n Wraps errors related to counters\n\nHistogramError(histogram.HistogramError)\n Wraps errors related to counters\n\nSynonyms:\nthemis.StoreError\nthemis StoreError","url":"themis.html#StoreError"},{"doc":"themis","title":"is_metric_name_used","content":"pub fn is_metric_name_used(\n  store: Store,\n  name: MetricName,\n) -&gt; Bool\n\nSynonyms:\nthemis.is_metric_name_used\nthemis is_metric_name_used","url":"themis.html#is_metric_name_used"},{"doc":"themis","title":"new","content":"pub fn new() -&gt; Store\n Creates a new empty metrics store.\nSynonyms:\nthemis.new\nthemis new","url":"themis.html#new"},{"doc":"themis","title":"print","content":"pub fn print(metrics_store store: Store) -&gt; String\n Formats all metrics in the store as a Prometheus-compatible text string.\n\n ## Examples\n\n \n let metrics_text = print(store)\n // # HELP my_metric My first gauge\n // # TYPE my_metric gauge\n // my_metric{foo=&quot;bar&quot;} 10\n // my_metric{toto=&quot;tata&quot;,wibble=&quot;wobble&quot;} +Inf\n \nSynonyms:\nthemis.print\nthemis print","url":"themis.html#print"},{"doc":"themis","title":"themis","content":"","url":"themis.html"},{"doc":"themis/counter","title":"increment","content":"pub fn increment(\n  store store: Store,\n  counter_name name_string: String,\n  labels labels_dict: Dict(String, String),\n) -&gt; Result(Store, StoreError)\n Increments a counter record with the given labels by 1.\n\n ## Arguments\n\n - `store`: The metrics store containing the counter\n - `counter_name`: The name of the counter\n - `labels`: A dictionary of labels\n\n ## Examples\n\n \n let labels = dict.from_list([#(&quot;instance&quot;, &quot;localhost:9090&quot;)])\n let assert Ok(store) = increment(\n   store,\n   &quot;http_request_failures&quot;,\n   labels,\n )\n \nSynonyms:\nthemis/counter.increment\nthemis/counter increment","url":"themis/counter.html#increment"},{"doc":"themis/counter","title":"increment_by","content":"pub fn increment_by(\n  store store: Store,\n  counter_name name_string: String,\n  labels labels_dict: Dict(String, String),\n  by by: Number,\n) -&gt; Result(Store, StoreError)\n Increments a counter record with the given labels by a specified value.\n\n ## Arguments\n\n - `store`: The metrics store containing the counter\n - `counter_name`: The name of the counter\n - `labels`: A dictionary of labels\n\n ## Examples\n\n \n let labels = dict.from_list([#(&quot;instance&quot;, &quot;localhost:9090&quot;)])\n let assert Ok(store) = increment(\n   store,\n   &quot;http_request_failures&quot;,\n   labels,\n   number.integer(12),\n )\n \nSynonyms:\nthemis/counter.increment_by\nthemis/counter increment_by","url":"themis/counter.html#increment_by"},{"doc":"themis/counter","title":"init_record","content":"pub fn init_record(\n  store store: Store,\n  counter_name name_string: String,\n  labels labels_dict: Dict(String, String),\n) -&gt; Result(Store, StoreError)\n Initializes a new counter metric record with the given labels.\n\n ## Arguments\n\n - `store`: The metrics store containing the counter\n - `counter_name`: The name of the counter\n - `labels`: A dictionary of labels\n\n ## Examples\n\n \n let labels = dict.from_list([#(&quot;instance&quot;, &quot;localhost:9090&quot;)])\n let assert Ok(store) = init_record(\n   store,\n   &quot;http_request_failures&quot;,\n   labels,\n )\n \nSynonyms:\nthemis/counter.init_record\nthemis/counter init_record","url":"themis/counter.html#init_record"},{"doc":"themis/counter","title":"register","content":"pub fn register(\n  store store: Store,\n  name name_string: String,\n  description description: String,\n) -&gt; Result(Store, StoreError)\n Registers a new counter metric to the store.\n\n ## Arguments\n\n - `store`: The metrics store to add the counter to\n - `name`: The name of the new counter metric (must be a valid Prometheus metric name)\n - `description`: A human-readable description of what the counter observes\n\n ## Examples\n\n \n let assert Ok(store) = register(\n   store,\n   &quot;http_request_failures&quot;,\n   &quot;Count of HTTP request failures&quot;,\n )\n \nSynonyms:\nthemis/counter.register\nthemis/counter register","url":"themis/counter.html#register"},{"doc":"themis/counter","title":"unregister","content":"pub fn unregister(\n  store store: Store,\n  counter_name name_string: String,\n) -&gt; Result(Store, StoreError)\n Removes a counter metric and all its recorded values from the store.\n\n ## Arguments\n\n - `store`: The metrics store containing the counter\n - `counter_name`: The name of the counter to delete\n\n ## Examples\n\n \n let assert Ok(store) = unregister(store, &quot;http_request_failures&quot;)\n \nSynonyms:\nthemis/counter.unregister\nthemis/counter unregister","url":"themis/counter.html#unregister"},{"doc":"themis/counter","title":"themis/counter","content":"","url":"themis/counter.html"},{"doc":"themis/gauge","title":"observe","content":"pub fn observe(\n  store store: Store,\n  gauge_name name_string: String,\n  labels labels_dict: Dict(String, String),\n  value value: Number,\n) -&gt; Result(Store, StoreError)\n Records a value for a gauge metric with the given labels.\n\n ## Arguments\n\n - `store`: The metrics store containing the gauge\n - `gauge_name`: The name of the gauge\n - `labels`: A dictionary of labels\n - `value`: The numeric value to record\n\n ## Examples\n\n \n let labels = dict.from_list([#(&quot;instance&quot;, &quot;localhost:9090&quot;)])\n let assert Ok(store) = insert_gauge_record(\n   store,\n   &quot;process_cpu_seconds_total&quot;,\n   labels,\n   int(42),\n )\n \nSynonyms:\nthemis/gauge.observe\nthemis/gauge observe","url":"themis/gauge.html#observe"},{"doc":"themis/gauge","title":"register","content":"pub fn register(\n  store store: Store,\n  name name_string: String,\n  description description: String,\n) -&gt; Result(Store, StoreError)\n Registers a new gauge metric to the store.\n\n ## Arguments\n\n - `store`: The metrics store to add the gauge to\n - `name`: The name of the new gauge metric (must be a valid Prometheus metric name)\n - `description`: A human-readable description of what the gauge observes\n\n ## Examples\n\n \n let assert Ok(store) = register(\n   store,\n   &quot;process_cpu_seconds_total&quot;,\n   &quot;Total user and system CPU time spent in seconds&quot;,\n )\n \nSynonyms:\nthemis/gauge.register\nthemis/gauge register","url":"themis/gauge.html#register"},{"doc":"themis/gauge","title":"unregister","content":"pub fn unregister(\n  store store: Store,\n  gauge_name name_string: String,\n) -&gt; Result(Store, StoreError)\n Removes a gauge metric and all its recorded values from the store.\n\n ## Arguments\n\n - `store`: The metrics store containing the gauge\n - `gauge_name`: The name of the gauge to delete\n\n ## Examples\n\n \n let assert Ok(store) = unregister(store, &quot;process_cpu_seconds_total&quot;)\n \nSynonyms:\nthemis/gauge.unregister\nthemis/gauge unregister","url":"themis/gauge.html#unregister"},{"doc":"themis/gauge","title":"themis/gauge","content":"","url":"themis/gauge.html"},{"doc":"themis/histogram","title":"init_record","content":"pub fn init_record(\n  store store: Store,\n  histogram_name name_string: String,\n  labels labels_dict: Dict(String, String),\n) -&gt; Result(Store, StoreError)\n Initializes a new histogram metric record with the given labels.\n\n ## Arguments\n\n - `store`: The metrics store containing the histogram\n - `histogram_name`: The name of the histogram\n - `labels`: A dictionary of labels\n - `bucket_boundaries`: A set of numbers defining the buckets by their `le` limits\n \n Note that a `+Inf` bucket is always added by default if it isn&#39;t provided.\n\n ## Examples\n\n \n let labels = dict.from_list([#(&quot;instance&quot;, &quot;localhost:9090&quot;)])\n let buckets = set.from_list(\n   [\n     number.decimal(0.05), \n     number.decimal(0.1), \n     number.decimal(0.2), \n     number.decimal(0.5),\n     number.integer(1),\n   ]\n )\n let assert Ok(store) = init_record(\n   store,\n   &quot;app_request_processing_seconds&quot;,\n   labels,\n   buckets,\n )\n \nSynonyms:\nthemis/histogram.init_record\nthemis/histogram init_record","url":"themis/histogram.html#init_record"},{"doc":"themis/histogram","title":"observe","content":"pub fn observe(\n  store store: Store,\n  histogram_name name_string: String,\n  labels labels_dict: Dict(String, String),\n  value value: Number,\n) -&gt; Result(Store, StoreError)\n Records an observation value for a histogram metric.\n\n ## Arguments\n\n - `store`: The metrics store containing the histogram\n - `histogram_name`: The name of the histogram metric\n - `labels`: A dictionary of labels\n - `value`: The value to record in the histogram\n\n ## Examples\n\n \n let labels = dict.from_list([#(&quot;instance&quot;, &quot;localhost:9090&quot;)])\n let value = number.decimal(0.123)\n let assert Ok(store) = observe(\n   store,\n   &quot;app_request_processing_seconds&quot;, \n   labels,\n   value,\n )\n \nSynonyms:\nthemis/histogram.observe\nthemis/histogram observe","url":"themis/histogram.html#observe"},{"doc":"themis/histogram","title":"register","content":"pub fn register(\n  store store: Store,\n  name name_string: String,\n  description description: String,\n  buckets buckets: Set(Number),\n) -&gt; Result(Store, StoreError)\n Registers a new histogram metric to the store.\n\n ## Arguments\n\n - `store`: The metrics store to add the histogram to\n - `name`: The name of the new histogram metric (must be a valid Prometheus metric name)\n - `description`: A human-readable description of what the histogram observes\n - `buckets`: A set of numbers defining the buckets by their `le` limits\n\n ## Examples\n\n \n let assert Ok(store) = register(\n   store,\n   &quot;app_request_processing_seconds&quot;,\n   &quot;Time spent processing request&quot;,\n )\n \nSynonyms:\nthemis/histogram.register\nthemis/histogram register","url":"themis/histogram.html#register"},{"doc":"themis/histogram","title":"unregister","content":"pub fn unregister(\n  store store: Store,\n  histogram_name name_string: String,\n) -&gt; Result(Store, StoreError)\n Unregisters a new histogram metric to the store.\n\n ## Arguments\n\n - `store`: The metrics store to add the histogram to\n - `name`: The name of the new histogram metric (must be a valid Prometheus metric name)\n\n ## Examples\n\n \n let assert Ok(store) = unregister(\n   store,\n   &quot;app_request_processing_seconds&quot;,\n )\n \nSynonyms:\nthemis/histogram.unregister\nthemis/histogram unregister","url":"themis/histogram.html#unregister"},{"doc":"themis/histogram","title":"themis/histogram","content":"","url":"themis/histogram.html"}]);